@@ -1,32 +1,40 @@
-use 5.008_000;
+## no critic (Modules::RequireExplicitPackage, InputOutput::RequireBriefOpen)
+
+use 5.008;
 use strict;
 use warnings;
 
 package SVN::Look;
-{
-  $SVN::Look::VERSION = '0.38';
-}
-# ABSTRACT: A caching wrapper around the svnlook command.
-
+# ABSTRACT: Caching wrapper around the svnlook command
+$SVN::Look::VERSION = '0.42';
 use Carp;
 use File::Spec::Functions;
 use List::MoreUtils qw{uniq};
 
 
+my @SVN_VERSION;
+
 BEGIN {
     my $path = $ENV{PATH} || '';
+
+    # Perl on Windows doesn't support the piped three-arg open. So we use
+    # the two-arg open here and don't care about Perl::Critic since this is
+    # a fixed string command.
+
     open my $svnlook, 'svnlook --version |' ## no critic (InputOutput::ProhibitTwoArgOpen)
-	or die "Aborting because I couldn't find the 'svnlook' executable in PATH='$path'.\n";
+        or die "Aborting because I couldn't find the 'svnlook' executable in PATH='$path'.\n";
     $_ = <$svnlook>;
-    if (my ($major, $minor, $patch) = (/(\d+)\.(\d+)\.(\d+)/)) {
-	$major > 1 || $major == 1 && $minor >= 4
-	    or die "I need at least version 1.4.0 of svnlook but you have only $major.$minor.$patch in PATH='$path'.\n";
+    if (@SVN_VERSION = (/(\d+)\.(\d+)\.(\d+)/)) {
+        if ($SVN_VERSION[0] < 1 || $SVN_VERSION[0] == 1 && $SVN_VERSION[1] < 4) {
+            die "I need at least version 1.4.0 of svnlook but you have only ",
+                join('.', @SVN_VERSION), " in PATH='$path'.\n";
+        }
     } else {
-	die "Can't grok Subversion version from svnlook --version command.\n";
+        die "Can't grok Subversion version from svnlook --version command.\n";
     }
-    local $/ = undef;		# slurp mode
+    local $/ = undef;           # slurp mode
     <$svnlook>;
-    close $svnlook or die "Can't close svnlook commnand.\n";
+    close $svnlook or die "Can't close svnlook command.\n";
 }
 
 
@@ -44,18 +52,52 @@ sub _svnlook {
     my ($self, $cmd, @args) = @_;
     my @cmd = ('svnlook', $cmd, $self->{repo});
     push @cmd, @{$self->{opts}} unless $cmd =~ /^(?:youngest|uuid|lock)$/;
-    my $args = @args ? '"' . join('" "', @args) . '"' : '';
-    open my $fd, join(' ', @cmd, $args, '|') ## no critic (InputOutput::ProhibitTwoArgOpen)
-        or die "Can't exec svnlook $cmd: $!\n";
+
+    my $fd;
+    my $tmpfile;
+    if ($^O ne 'MSWin32') {
+        open $fd, '-|', @cmd, @args
+            or die "Can't exec svnlook $cmd: $!\n";
+    } else {
+        # Windows doesn't support the three-argument version of open
+        # neither the pipe function. So we run the svnlook command
+        # with system, sending its output to a temporary file and
+        # opening the file later in $fd.
+
+        # Create the temporary file.
+        require File::Temp;
+        $tmpfile = File::Temp->new();
+        my $filename = $tmpfile->filename;
+
+        ## no critic (ProhibitTwoArgOpen, ProhibitBarewordFileHandles)
+
+        # Dup STDOUT and direct it to the file
+        no warnings 'once';     ## no critic (TestingAndDebugging::ProhibitNoWarnings)
+        open OLDOUT, '>&STDOUT'   or die "Can't dup STDOUT: $!\n";
+        open STDOUT, ">$filename" or die "Can't redirect STDOUT to $filename: $!\n";
+
+        # Shell out the svnlook command
+        system(@cmd, @args) == 0
+            or die "system @cmd failed: $?\n";
+
+        # Restore STDOUT
+        open STDOUT, '>&OLDOUT' or die "Can't redirect STDOUT to its former value: $!\n";
+
+        # Open the temporary file
+        open $fd, "<$filename" or die "Can't open $filename: $!\n";
+
+        ## use critic
+    }
+
     if (wantarray) {
         my @lines = <$fd>;
         unless (close $fd) {
-	    if ($!) {
-		die "Error closing (wantarray) svnlook $cmd pipe: $!\n";
-	    } else {
-		die "Exit status $? from (wantarray) svnlook $cmd\n";
-	    }
-	}
+            if ($!) {
+                die "Error closing (wantarray) svnlook $cmd pipe: $!\n";
+            } else {
+                die "Exit status $? from (wantarray) svnlook $cmd\n";
+            }
+        }
         chomp @lines;
         return @lines;
     }
@@ -63,12 +105,12 @@ sub _svnlook {
         local $/ = undef;
         my $line = <$fd>;
         unless (close $fd) {
-	    if ($!) {
-		die "Error closing svnlook $cmd pipe: $!\n";
-	    } else {
-		die "Exit status $? from svnlook $cmd\n";
-	    }
-	}
+            if ($!) {
+                die "Error closing svnlook $cmd pipe: $!\n";
+            } else {
+                die "Exit status $? from svnlook $cmd\n";
+            }
+        }
         chomp $line unless $cmd eq 'cat';
         return $line;
     }
@@ -115,7 +157,7 @@ sub changed_hash {
         foreach ($self->_svnlook('changed', '--copy-info')) {
             next if length($_) <= 4;
             chomp;
-            my ($action, $prop, undef, undef, $changed) = unpack 'AAAA A*', $_;
+            my ($action, $prop, $changed) = unpack 'AAxx a*', $_;
             if    ($action eq 'A') {
                 push @added,   $changed;
             }
@@ -175,9 +217,9 @@ sub changed {
     my $hash = $self->changed_hash();
     unless (exists $hash->{changed}) {
         $hash->{changed} = [sort(uniq(@{$hash->{added}},
-				      @{$hash->{updated}},
-				      @{$hash->{deleted}},
-				      @{$hash->{prop_modified}}))];
+                                      @{$hash->{updated}},
+                                      @{$hash->{deleted}},
+                                      @{$hash->{prop_modified}}))];
     }
     return @{$hash->{changed}};
 }
@@ -232,21 +274,21 @@ sub info {
 }
 
 
-sub lock {
+sub lock {                      ## no critic (ProhibitBuiltinHomonyms)
     my ($self, $path) = @_;
     my %lock = ();
     my @lock = $self->_svnlook('lock', $path);
 
     while (my $line = shift @lock) {
-	chomp $line;
-	my ($key, $value) = split /:\s*/, $line, 2;
-	if ($key =~ /^Comment/) {
-	    $lock{Comment} = join('', @lock);
-	    last;
-	}
-	else {
-	    $lock{$key} = $value;
-	}
+        chomp $line;
+        my ($key, $value) = split /:\s*/, $line, 2;
+        if ($key =~ /^Comment/) {
+            $lock{Comment} = join('', @lock);
+            last;
+        }
+        else {
+            $lock{$key} = $value;
+        }
     }
 
     return %lock ? \%lock : undef;
@@ -268,14 +310,46 @@ sub propget {
 }
 
 
+# The 'svnlook proplist' command had its output format changed in svn
+# 1.8.0. So, in order to make the code more stable we try to use the
+# --xml option. However, this option was implemented on svn 1.6.0
+# only. Since we still want to support older Subversions we have to
+# check if we can use that option first. See discussion on
+# https://github.com/gnustavo/SVN-Look/pull/1.
+
+my $proplist_does_support_xml_option;
+
 sub proplist {
     my ($self, $path) = @_;
+
     unless ($self->{proplist}{$path}) {
-        my $text = $self->_svnlook('proplist', '--verbose', $path);
-        my @list = split /^\s\s(\S+)\s:\s/m, $text;
-        shift @list;            # skip the leading empty field
-        chomp(my %hash = @list);
-        $self->{proplist}{$path} = \%hash;
+        $proplist_does_support_xml_option = $SVN_VERSION[0] > 1 || $SVN_VERSION[0] == 1 && $SVN_VERSION[1] >= 8
+            unless defined $proplist_does_support_xml_option;
+
+        if ($proplist_does_support_xml_option) {
+            my $xml = $self->_svnlook(qw/proplist --verbose --xml/, $path);
+            require XML::Simple;
+            my $dom = XML::Simple::XMLin($xml, ForceArray => ['property']);
+            while (my ($prop, $value) = each %{$dom->{target}{property}}) {
+                my $content = $value->{content};
+                if (my $encoding = $value->{encoding}) {
+                    if ($encoding eq 'base64') {
+                        require MIME::Base64;
+                        $content = MIME::Base64::decode($content);
+                    } else {
+                        die "Don't know how to decode property '$prop' value encoded as '$encoding'\n";
+                    }
+                }
+                $self->{proplist}{$path}{$prop} = $content;
+            }
+        } else {
+            # Old syntax up to SVN 1.7.
+            my $text = $self->_svnlook('proplist', '--verbose', $path);
+            my @list = split /^\s\s(\S+)\s:\s/m, $text;
+            shift @list;        # skip the leading empty field
+            chomp(my %hash = @list);
+            $self->{proplist}{$path} = \%hash;
+        }
     }
     return $self->{proplist}{$path};
 }
@@ -301,15 +375,18 @@ sub youngest {
 1; # End of SVN::Look
 
 __END__
+
 =pod
 
+=encoding UTF-8
+
 =head1 NAME
 
-SVN::Look - A caching wrapper around the svnlook command.
+SVN::Look - Caching wrapper around the svnlook command
 
 =head1 VERSION
 
-version 0.38
+version 0.42
 
 =head1 SYNOPSIS
 
@@ -332,7 +409,7 @@ used to gather all sorts of information about a repository, its
 revisions, and its transactions. This module provides a simple object
 oriented interface to a specific svnlook invocation, to make it easier
 to hook writers to get and use the information they need. Moreover,
-all the information gathered buy calling the svnlook command is cached
+all the information gathered by calling the svnlook command is cached
 in the object, avoiding repetitious calls.
 
 =head1 METHODS
@@ -551,16 +628,19 @@ Returns the repository's UUID.
 
 Returns the repository's youngest revision number.
 
+=head1 REPOSITORY
+
+L<https://github.com/gnustavo/SVN-Look>
+
 =head1 AUTHOR
 
 Gustavo L. de M. Chaves <gnustavo@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2012 by CPqD.
+This software is copyright (c) 2020 by CPqD.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
 
 =cut
-
